@@ -514,7 +514,8 @@ the length of that format.")
                  (oci-stmt-prepare (deref-vp stmthp)
                                    (deref-vp errhp)
                                    c-stmt-string
-                                   (length sql-stmt-string)
+                                   (uffi:foreign-string-length length
+                                                               sql-stmt-string)
                                    +oci-ntv-syntax+ +oci-default+ :database db)
                  (oci-attr-get (deref-vp stmthp)
                                +oci-htype-stmt+
